"use strict";
var ts = require("typescript");
var ApiPackage_1 = require("./definitions/ApiPackage");
var TypeScriptHelpers_1 = require("./TypeScriptHelpers");
var DocItemLoader_1 = require("./DocItemLoader");
/**
  * The main entry point for the "api-extractor" utility.  The Analyzer object invokes the
  * TypeScript Compiler API to analyze a project, and constructs the ApiItem
  * abstract syntax tree.
  */
var Analyzer = (function () {
    function Analyzer(errorHandler) {
        this.errorHandler = errorHandler || Analyzer.defaultErrorHandler;
    }
    /**
      * The default implementation of ApiErrorHandler, which merely writes to console.log().
      */
    Analyzer.defaultErrorHandler = function (message, fileName, lineNumber) {
        console.log("ERROR: [" + fileName + ":" + lineNumber + "] " + message);
    };
    /**
      * Analyzes the specified project.
      */
    Analyzer.prototype.analyze = function (options) {
        this.docItemLoader = new DocItemLoader_1.default(options.compilerOptions.rootDir);
        var rootFiles = [options.entryPointFile].concat(options.otherFiles || []);
        var program = ts.createProgram(rootFiles, options.compilerOptions);
        // This runs a full type analysis, and then augments the Abstract Syntax Tree (i.e. declarations)
        // with semantic information (i.e. symbols).  The "diagnostics" are a subset of the everyday
        // compile errors that would result from a full compilation.
        for (var _i = 0, _a = program.getSemanticDiagnostics(); _i < _a.length; _i++) {
            var diagnostic = _a[_i];
            this.reportError('TypeScript: ' + diagnostic.messageText, diagnostic.file, diagnostic.start);
        }
        this.typeChecker = program.getTypeChecker();
        var rootFile = program.getSourceFile(options.entryPointFile);
        if (!rootFile) {
            throw new Error('Unable to load file: ' + options.entryPointFile);
        }
        var rootFileSymbol = TypeScriptHelpers_1.default.getSymbolForDeclaration(rootFile);
        this.package = new ApiPackage_1.default(this, rootFileSymbol);
    };
    /**
      * Reports an error message to the registered ApiErrorHandler.
      */
    Analyzer.prototype.reportError = function (message, sourceFile, start) {
        var lineNumber = sourceFile.getLineAndCharacterOfPosition(start).line;
        this.errorHandler(message, sourceFile.fileName, lineNumber);
    };
    return Analyzer;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Analyzer;

//# sourceMappingURL=Analyzer.js.map
