"use strict";
var ApiDocumentation_1 = require("./definitions/ApiDocumentation");
var Token_1 = require("./Token");
var DocElementParser = (function () {
    function DocElementParser() {
    }
    DocElementParser.makeTextElement = function (text) {
        if (!text) {
            return;
        }
        return { kind: 'textDocElement', value: text };
    };
    DocElementParser.parse = function (tokenizer, reportError) {
        var docElements = [];
        var parsing = true;
        var token;
        while (parsing) {
            token = tokenizer.peekToken();
            if (!token) {
                parsing = false; // end of stream
                break;
            }
            if (token.type === Token_1.TokenType.Tag) {
                switch (token.tag) {
                    case '@see':
                        tokenizer.getToken();
                        docElements.push({
                            kind: 'seeDocElement',
                            seeElements: this.parse(tokenizer, reportError)
                        });
                        break;
                    default:
                        parsing = false; // end of summary tokens
                        break;
                }
            }
            else if (token.type === Token_1.TokenType.Inline) {
                switch (token.tag) {
                    case '@link':
                        var linkDocElement = this.parseLinkTag(token, reportError);
                        if (linkDocElement) {
                            docElements.push(linkDocElement);
                        }
                        tokenizer.getToken(); // get the link token
                        break;
                    default:
                        parsing = false;
                        break;
                }
            }
            else if (token.type === Token_1.TokenType.Text) {
                docElements.push({ kind: 'textDocElement', value: token.text });
                tokenizer.getToken();
            }
            else {
                reportError("Unidentifiable Token " + token.type + " " + token.tag + " " + token.text);
            }
        }
        return docElements;
    };
    /**
     * This method parses the semantic information in an \@link JSDoc tag, creates and returns a
     * linkDocElement with the corresponding information. If the corresponding inline tag \@link is
     * not formatted correctly an error will be reported.
     *
     * The format for the \@link tag is {\@link url or API defintion reference | display text}, where
     * the '|' is only needed if the optional display text is given.
     *
     * Examples:
     * \{@link http://microsoft.com | microsoft home \}
     * \{@link http://microsoft.com \}
     * \{@link @microsoft/sp-core-library:Guid.newGuid | new Guid Object \}
     * \{@link @microsoft/sp-core-library:Guid.newGuid \}
     */
    DocElementParser.parseLinkTag = function (tokenItem, reportError) {
        if (!tokenItem.text) {
            reportError('Invalid @link inline token, a url or API definition reference must be given');
            return;
        }
        // Make sure there are no extra pipes
        var pipeSplitContent = tokenItem.text.split('|');
        pipeSplitContent = pipeSplitContent.map(function (value) {
            if (value) {
                return value.trim();
            }
        });
        if (pipeSplitContent.length > 2) {
            reportError('Invalid @link parameters, at most pipe character allowed.');
            return;
        }
        // Try to guess if the tokenContent is a link or API definition reference
        var linkDocElement;
        if (tokenItem.text.match(this._hrefRegEx)) {
            var urlContent = pipeSplitContent[0].split(' ');
            // Make sure only a single url is given
            if (urlContent.length > 1 && urlContent[1] !== '') {
                reportError('Invalid @link parameter, url must be a single string.');
                return;
            }
            linkDocElement = {
                kind: 'linkDocElement',
                referenceType: 'href',
                targetUrl: urlContent[0],
                value: ''
            };
        }
        else {
            // we are processing an API definition reference
            var apiDefitionRef = ApiDocumentation_1.default.parseApiReferenceExpression(pipeSplitContent[0], reportError);
            // Once we can locate local API definitions, an error should be reported here if not found.
            if (apiDefitionRef) {
                linkDocElement = {
                    kind: 'linkDocElement',
                    referenceType: 'code',
                    scopeName: apiDefitionRef.scopeName,
                    packageName: apiDefitionRef.packageName,
                    exportName: apiDefitionRef.exportName,
                    memberName: apiDefitionRef.memberName
                };
            }
        }
        // If a display name is given, ensure it only contains characters for words.
        if (linkDocElement && pipeSplitContent.length > 1) {
            var displayTextParts = pipeSplitContent[1].match(this._wordRegEx);
            if (displayTextParts && displayTextParts[0].length !== pipeSplitContent[1].length) {
                reportError('Display name in @link token may only contain alphabetic characters.');
                return;
            }
            // Full match is valid text
            linkDocElement.value = displayTextParts[0].trim();
        }
        return linkDocElement;
    };
    return DocElementParser;
}());
/**
 * Matches only strings that contain characters for words.
 * Any non word characters or spaces, will be present in the third entry in the match results
 * if they exist.
 */
DocElementParser._wordRegEx = /^([\w\s]*)/;
/**
 * Matches a href reference. This is used to get an idea whether a given reference is for an href
 * or an API definition reference.
 *
 * For example, the following would be matched:
 * 'http://'
 * 'https://'
 *
 * The following would not be matched:
 * '@microsoft/sp-core-library:Guid.newGuid'
 * 'Guid.newGuid'
 * 'Guid'
 */
DocElementParser._hrefRegEx = /^[a-z]+:\/\//;
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = DocElementParser;

//# sourceMappingURL=DocElementParser.js.map
