"use strict";
var fsx = require("fs-extra");
var os = require("os");
var path = require("path");
var JsonFile_1 = require("./JsonFile");
/**
 * A loader for locating the IDocItem associated with a given project and API item.
 * The DocItem loader utilizes the json files generated by the API-Extractor ApiJsonGenerator.
 * The IDocItem can then be used to enforce correct API usage, like enforcing internal.
 * To use to DocItemLoader: provide a projectFolder to construct a instance of the DocItemLoader,
 * then use DocItemLoader.getItem to retrieve the IDocItem of a particular API item.
 */
var DocItemLoader = (function () {
    /**
     * The projectFolder is the top-level folder containing package.json for a project
     * that we are compiling.
     */
    function DocItemLoader(projectFolder) {
        if (!fsx.existsSync(path.join(projectFolder, 'package.json'))) {
            throw new Error("An NPM project was not found in the specified folder: " + projectFolder);
        }
        this.projectFolder = projectFolder;
        this.cache = new Map();
    }
    /**
     * Attempts to retrieve an API item from the provided apiDefinitionRef.
     * First checks the cache for the package, if not in the cache then the method
     * will attempt to the locate the associated json file to load the package and
     * check there. If the API item can not be found the method will return undefined.
     */
    DocItemLoader.prototype.getItem = function (apiDefinitionRef) {
        if (!apiDefinitionRef) {
            throw new Error('Expected param to DocItemLoader.getItem() to be defined');
        }
        // Try to load the package given the provided packageName into the cache
        var docPackage = this.getPackage(apiDefinitionRef);
        // Check if package was not found
        if (!docPackage) {
            return undefined;
        }
        if (apiDefinitionRef.exportName in docPackage.exports) {
            var docItem = docPackage.exports[apiDefinitionRef.exportName];
            // If memberName exists then check for the existense of the name
            if (apiDefinitionRef.memberName) {
                if (apiDefinitionRef.memberName in docItem.members) {
                    docItem = docItem.members[apiDefinitionRef.memberName];
                }
                else {
                    // member name was not found, apiDefinitionRef is invalid
                    return undefined;
                }
            }
            // Correct doc item was found
            return docItem;
        }
        else {
            // Not found
            return undefined;
        }
    };
    /**
     * Attempts to locate and load the IDocPackage object from the project folder's
     * node modules. If the package already exists in the cache, nothing is done.
     *
     * @param apiDefinitionRef - interface with propropties pertaining to the API definition reference
     */
    DocItemLoader.prototype.getPackage = function (apiDefinitionRef) {
        var cachePackageName = '';
        // We concatenate the scopeName and packageName in case there are packageName conflicts
        if (apiDefinitionRef.scopeName) {
            cachePackageName = apiDefinitionRef.scopeName + "/" + apiDefinitionRef.packageName;
        }
        else {
            cachePackageName = apiDefinitionRef.packageName;
        }
        // Check if package exists in cache
        if (cachePackageName in this.cache) {
            return this.cache[cachePackageName];
        }
        // Doesn't exist in cache, attempt to load the json file
        var packageJsonFilePath = path.join(this.projectFolder, 'node_modules', apiDefinitionRef.scopeName, apiDefinitionRef.packageName, "dist/" + apiDefinitionRef.packageName + ".api.json");
        if (!fsx.existsSync(path.join(packageJsonFilePath))) {
            return undefined;
        }
        var apiPackage = JsonFile_1.default.loadJsonFile(packageJsonFilePath);
        // Validate that the output conforms to our JSON schema
        var apiJsonSchema = JsonFile_1.default.loadJsonFile(path.join(__dirname, './schemas/api-json-schema.json'));
        JsonFile_1.default.validateSchema(apiPackage, apiJsonSchema, function (errorDetail) {
            var errorMessage = "ApiJsonGenerator validation error - output does not conform to api-json-schema.json:" + os.EOL
                + errorDetail;
            console.log(os.EOL + 'ERROR: ' + errorMessage + os.EOL + os.EOL);
            throw new Error(errorMessage);
        });
        // JSON schema has been found and validated
        this.cache[cachePackageName] = apiPackage;
        return apiPackage;
    };
    return DocItemLoader;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = DocItemLoader;

//# sourceMappingURL=DocItemLoader.js.map
