/// <reference types="es6-collections" />
import ApiItem from './ApiItem';
import { IDocElement, IParam } from '../IDocElement';
import DocItemLoader from '../DocItemLoader';
import { IApiDefinitionReference } from '../IApiDefinitionReference';
import Token from '../Token';
import Tokenizer from '../Tokenizer';
/**
  * An "API Tag" is a custom JSDoc tag which indicates whether an ApiItem definition
  * is considered Public API for third party developers, as well as its release
  * stage (alpha, beta, etc).
  * @see https://onedrive.visualstudio.com/DefaultCollection/SPPPlat/_git/sp-client
  *      ?path=/common/docs/ApiPrinciplesAndProcess.md
  */
export declare enum ApiTag {
    /**
     * No API Tag was specified in the JSDoc summary.
     */
    None = 0,
    /**
     * The API was documented as internal, i.e. not callable by third party developers.
     */
    Internal = 1,
    /**
     * The API was documented as "alpha."  This status is not generally used.  See the
     * ApiPrinciplesAndProcess.md for details.
     */
    Alpha = 2,
    /**
     * The API was documented as callable by third party developers, but at their own risk.
     * Web parts that call beta APIs should only be used for experimentation, because the Web Part
     * will break if Microsoft changes the API signature later.
     */
    Beta = 3,
    /**
     * The API was documented as callable by third party developers, with a guarantee that Microsoft will
     * never make any breaking changes once the API is published.
     */
    Public = 4,
}
/**
 * A scope and package name are semantic information within an API reference expression.
 */
export interface IScopePackageName {
    /**
     * The scope name of an API reference expression.
     */
    scope: string;
    /**
     * The package name of an API reference expression.
     */
    package: string;
}
export default class ApiDocumentation {
    private static _allowedRegularJsdocTags;
    private static _allowedInlineJsdocTags;
    /**
     * Match JsDoc block tags and inline tags
     * Example "@a @b@c d@e @f {whatever} {@link a} { @something } \@g" => ["@a", "@f", "{@link a}", "{ @something }"]
     */
    private static _jsdocTagsRegex;
    /**
     * Splits an API reference expression into two parts, first part is the scopename/packageName and
     * the second part is the exportName.memberName.
     */
    private static _packageRegEx;
    /**
     * Splits the exportName.memberName into two respective parts.
     */
    private static _memberRegEx;
    /**
     * Used to ensure that the export name contains only text characters.
     */
    private static _exportRegEx;
    /**
     * Corresponding ApiItem for this documentation object
     */
    apiItem: ApiItem;
    /**
     * This returns the JSDoc comment block for the given item (without "/**" characters),
     * or an empty string if there is no comment.  This propery will never be undefined.
     *
     * Example: "Example Function\n@returns the number of items\n@internal"
     */
    docComment: string;
    /**
     * The docComment text string split into an array of ITokenItems.  The tokens are essentially either
     * JSDoc tags (which start with the "@" character) or substrings containing the
     * remaining text.  The array can be empty, but not undefined.
     * Example:
     * docComment       = "Example Function\n@returns the number of items\n@internal  "
     * docCommentTokens = [
     *  {tokenType: 'text', parameter: 'Example Function\n'},
     *  {tokenType: '\@returns', parameter: ''}
     *  {tokenType: 'text', parameter: 'the number of items\n'}
     *  {tokenType: '@internal', parameter: ''}
     * ];
     */
    docCommentTokens: Token[];
    /**
    * docCommentTokens that are parsed into Doc Elements.
    */
    summary: IDocElement[];
    deprecatedMessage: IDocElement[];
    remarks: IDocElement[];
    returnsMessage: IDocElement[];
    parameters: Map<string, IParam>;
    /**
     * Indicates that this definition does not have adequate JSDoc comments.  If isMissing=true,
     * then this will be noted in the API file produced by ApiFileGenerator.  (The JSDoc
     * text itself is not included in that report, because documentation changes do not
     * require an API review, and thus should not cause a diff for that report.)
     */
    isMissing: boolean;
    /**
     * An "API Tag" is a custom JSDoc tag which indicates whether this definition
     * is considered Public API for third party developers, as well as its release
     * stage (alpha, beta, etc).
     */
    apiTag: ApiTag;
    /**
     * True if the "@preapproved" tag was specified.
     * Indicates that this internal API is exempt from further reviews.
     */
    preapproved?: boolean;
    deprecated?: string;
    internalremarks?: string;
    paramDocs?: Map<string, string>;
    returns?: string;
    see?: string[];
    isDocBeta?: boolean;
    isDocInherited?: boolean;
    isDocInheritedDeprecated?: boolean;
    isOverride?: boolean;
    readonly?: boolean;
    docItemLoader: DocItemLoader;
    reportError: (message: string) => void;
    /**
     * Takes an API reference expression of the form '@scopeName/packageName:exportName.memberName'
     * and deconstructs it into an IApiDefinitionReference interface object.
     */
    static parseApiReferenceExpression(apiReferenceExpr: string, reportError: (message: string) => void): IApiDefinitionReference;
    /**
     * For a scoped NPM package name this separates the scope and package parts.  For example:
     * parseScopedPackageName('@my-scope/myproject') = { scope: '@my-scope', package: 'myproject' }
     * parseScopedPackageName('myproject') = { scope: '', package: 'myproject' }
     */
    private static parseScopedPackageName(scopedName);
    constructor(apiItem: ApiItem, docItemLoader: DocItemLoader, errorLogger: (message: string) => void);
    protected _getJsDocs(apiItem: ApiItem): string;
    protected _parseDocs(): void;
    /**
     * This method parses the semantic information in an \@inheritdoc JSDoc tag and sets
     * all the relevant documenation properties from the inherited doc onto the documenation
     * of the current api item.
     *
     * The format for the \@inheritdoc tag is {\@inheritdoc scopeName/packageName:exportName.memberName}.
     * For more information on the format see IInheritdocRef.
     */
    protected _parseInheritDoc(token: Token): void;
    protected _parseParam(tokenizer: Tokenizer): IParam;
    private _reportBadJSDocTag(token);
    private _checkInheritDocStatus();
}
