/* tslint:disable:no-bitwise */
"use strict";
var ts = require("typescript");
var TypeScriptHelpers_1 = require("../TypeScriptHelpers");
var DocElementParser_1 = require("../DocElementParser");
var Token_1 = require("../Token");
var Tokenizer_1 = require("../Tokenizer");
/**
  * An "API Tag" is a custom JSDoc tag which indicates whether an ApiItem definition
  * is considered Public API for third party developers, as well as its release
  * stage (alpha, beta, etc).
  * @see https://onedrive.visualstudio.com/DefaultCollection/SPPPlat/_git/sp-client
  *      ?path=/common/docs/ApiPrinciplesAndProcess.md
  */
var ApiTag;
(function (ApiTag) {
    /**
     * No API Tag was specified in the JSDoc summary.
     */
    ApiTag[ApiTag["None"] = 0] = "None";
    /**
     * The API was documented as internal, i.e. not callable by third party developers.
     */
    ApiTag[ApiTag["Internal"] = 1] = "Internal";
    /**
     * The API was documented as "alpha."  This status is not generally used.  See the
     * ApiPrinciplesAndProcess.md for details.
     */
    ApiTag[ApiTag["Alpha"] = 2] = "Alpha";
    /**
     * The API was documented as callable by third party developers, but at their own risk.
     * Web parts that call beta APIs should only be used for experimentation, because the Web Part
     * will break if Microsoft changes the API signature later.
     */
    ApiTag[ApiTag["Beta"] = 3] = "Beta";
    /**
     * The API was documented as callable by third party developers, with a guarantee that Microsoft will
     * never make any breaking changes once the API is published.
     */
    ApiTag[ApiTag["Public"] = 4] = "Public";
})(ApiTag = exports.ApiTag || (exports.ApiTag = {}));
var ApiDocumentation = (function () {
    function ApiDocumentation(apiItem, docItemLoader, errorLogger) {
        this.apiItem = apiItem;
        this.docItemLoader = docItemLoader;
        this.reportError = errorLogger;
        this.docComment = this._getJsDocs(apiItem);
        this.parameters = new Map();
        this._parseDocs();
    }
    /**
     * Takes an API reference expression of the form '@scopeName/packageName:exportName.memberName'
     * and deconstructs it into an IApiDefinitionReference interface object.
     */
    ApiDocumentation.parseApiReferenceExpression = function (apiReferenceExpr, reportError) {
        if (!apiReferenceExpr || apiReferenceExpr.split(' ').length > 1) {
            reportError('API reference expression must be of the form: ' +
                '\'scopeName/packageName:exportName.memberName | display text\'' +
                'where the \'|\' is required if a display text is provided');
            return;
        }
        var apiDefitionRef = { scopeName: '', packageName: '', exportName: '', memberName: '' };
        var parts;
        // E.g. @microsoft/sp-core-library:Guid.equals
        parts = apiReferenceExpr.match(ApiDocumentation._packageRegEx);
        if (parts) {
            // parts[1] is of the form ‘@microsoft/sp-core-library’ or ‘sp-core-library’
            var scopePackageName = ApiDocumentation.parseScopedPackageName(parts[1]);
            apiDefitionRef.scopeName = scopePackageName.scope;
            apiDefitionRef.packageName = scopePackageName.package;
            apiReferenceExpr = parts[2]; // e.g. Guid.equals
        }
        // E.g. Guid.equals
        parts = apiReferenceExpr.match(ApiDocumentation._memberRegEx);
        if (parts) {
            apiDefitionRef.exportName = parts[1]; // e.g. Guid, can never be undefined
            apiDefitionRef.memberName = parts[2] ? parts[2] : ''; // e.g. equals
        }
        else {
            // the export name is required
            throw reportError("Api reference expression contains invalid characters: " + apiReferenceExpr);
        }
        if (!apiReferenceExpr.match(ApiDocumentation._exportRegEx)) {
            throw reportError("Api reference expression contains invalid characters: " + apiReferenceExpr);
        }
        return apiDefitionRef;
    };
    /**
     * For a scoped NPM package name this separates the scope and package parts.  For example:
     * parseScopedPackageName('@my-scope/myproject') = { scope: '@my-scope', package: 'myproject' }
     * parseScopedPackageName('myproject') = { scope: '', package: 'myproject' }
     */
    ApiDocumentation.parseScopedPackageName = function (scopedName) {
        if (scopedName.substr(0, 1) !== '@') {
            return { scope: '', package: scopedName };
        }
        var slashIndex = scopedName.indexOf('/');
        if (slashIndex >= 0) {
            return { scope: scopedName.substr(0, slashIndex), package: scopedName.substr(slashIndex + 1) };
        }
        else {
            throw new Error('Invalid scoped name: ' + scopedName);
        }
    };
    ApiDocumentation.prototype._getJsDocs = function (apiItem) {
        var sourceFile = apiItem.getDeclaration().getSourceFile();
        var jsDoc = TypeScriptHelpers_1.default.getJsDocComments(apiItem.getDeclaration(), this.reportError);
        // Eliminate tags and then count the English letters.  Are there at least 10 letters of text?
        // If not, we consider the definition to be "missingDocumentation".
        var condensedDocs = jsDoc
            .replace(ApiDocumentation._jsdocTagsRegex, '')
            .replace(/[^a-z]/gi, '');
        this.isMissing = apiItem.shouldHaveDocumentation() && condensedDocs.length <= 10;
        return jsDoc;
    };
    ApiDocumentation.prototype._parseDocs = function () {
        var tokenizer = new Tokenizer_1.default(this.docComment, this.reportError);
        this.summary = DocElementParser_1.default.parse(tokenizer, this.reportError);
        this.returnsMessage = [];
        this.deprecatedMessage = [];
        this.remarks = [];
        this.apiTag = ApiTag.None;
        var apiTagCount = 0;
        var parsing = true;
        while (parsing) {
            var token = tokenizer.peekToken();
            if (!token) {
                parsing = false; // end of stream
                // Report error if @inheritdoc is deprecated but no @deprecated tag present here
                if (this.isDocInheritedDeprecated && this.deprecatedMessage.length === 0) {
                    // if this documentation inherits docs from a deprecated API item, then
                    // this documentation must either have a deprecated message or it must
                    // not use the @inheritdoc and copy+paste the documentation
                    this.reportError("A deprecation message must be included after the @deprecated tag.");
                }
                break;
            }
            if (token.type === Token_1.TokenType.Tag) {
                switch (token.tag) {
                    case '@remarks':
                        tokenizer.getToken();
                        this._checkInheritDocStatus();
                        this.remarks = DocElementParser_1.default.parse(tokenizer, this.reportError);
                        break;
                    case '@returns':
                        tokenizer.getToken();
                        this._checkInheritDocStatus();
                        this.returnsMessage = DocElementParser_1.default.parse(tokenizer, this.reportError);
                        break;
                    case '@param':
                        tokenizer.getToken();
                        this._checkInheritDocStatus();
                        var param = this._parseParam(tokenizer);
                        if (param) {
                            this.parameters[param.name] = param;
                        }
                        break;
                    case '@deprecated':
                        tokenizer.getToken();
                        this.deprecatedMessage = DocElementParser_1.default.parse(tokenizer, this.reportError);
                        if (!this.deprecatedMessage || this.deprecatedMessage.length === 0) {
                            this.reportError("deprecated description required after @deprecated JSDoc tag.");
                        }
                        break;
                    case '@internalremarks':
                        // parse but discard
                        tokenizer.getToken();
                        DocElementParser_1.default.parse(tokenizer, this.reportError);
                        break;
                    case '@public':
                        tokenizer.getToken();
                        this.apiTag = ApiTag.Public;
                        ++apiTagCount;
                        break;
                    case '@internal':
                        tokenizer.getToken();
                        this.apiTag = ApiTag.Internal;
                        ++apiTagCount;
                        break;
                    case '@alpha':
                        tokenizer.getToken();
                        this.apiTag = ApiTag.Alpha;
                        ++apiTagCount;
                        break;
                    case '@beta':
                        tokenizer.getToken();
                        this.apiTag = ApiTag.Beta;
                        ++apiTagCount;
                        break;
                    case '@preapproved':
                        tokenizer.getToken();
                        this.preapproved = true;
                        break;
                    case '@readonly':
                        tokenizer.getToken();
                        this.readonly = true;
                        break;
                    case '@betadocumentation':
                        tokenizer.getToken();
                        this.isDocBeta = true;
                        break;
                    default:
                        tokenizer.getToken();
                        this._reportBadJSDocTag(token);
                }
            }
            else if (token.type === Token_1.TokenType.Inline) {
                switch (token.tag) {
                    case '@inheritdoc':
                        tokenizer.getToken();
                        if (this.summary.length > 0) {
                            this.reportError('Cannot provide summary in JsDoc if @inheritdoc tag is given');
                        }
                        this._parseInheritDoc(token);
                        this.isDocInherited = true;
                        break;
                    case '@link':
                        var linkDocElement = DocElementParser_1.default.parseLinkTag(tokenizer.getToken(), this.reportError);
                        if (linkDocElement) {
                            this.summary.push(linkDocElement);
                        }
                        break;
                    default:
                        tokenizer.getToken();
                        this._reportBadJSDocTag(token);
                        break;
                }
            }
            else if (token.type === Token_1.TokenType.Text) {
                tokenizer.getToken();
                // Shorten "This is too long text" to "This is..."
                var MAX_LENGTH = 40;
                var problemText = token.text.trim();
                if (problemText.length > MAX_LENGTH) {
                    problemText = problemText.substr(0, MAX_LENGTH - 3).trim() + '...';
                }
                this.reportError("Unexpected text in JSDoc comment: \"" + problemText + "\"");
            }
            else {
                tokenizer.getToken();
                // This would be a program bug
                this.reportError("Unexpected token: " + token.type + " " + token.tag + " " + token.text);
            }
        }
        if (apiTagCount > 1) {
            this.reportError('More than one API Tag was specified');
        }
        if (this.preapproved) {
            if (this.apiTag !== ApiTag.Internal) {
                this.reportError('The @preapproved tag may only be applied to @internal defintions');
                this.preapproved = false;
            }
            else if (!(this.apiItem.getDeclarationSymbol().flags & (ts.SymbolFlags.Interface | ts.SymbolFlags.Class))) {
                this.reportError('The @preapproved tag may only be applied to classes and interfaces');
                this.preapproved = false;
            }
        }
    };
    /**
     * This method parses the semantic information in an \@inheritdoc JSDoc tag and sets
     * all the relevant documenation properties from the inherited doc onto the documenation
     * of the current api item.
     *
     * The format for the \@inheritdoc tag is {\@inheritdoc scopeName/packageName:exportName.memberName}.
     * For more information on the format see IInheritdocRef.
     */
    ApiDocumentation.prototype._parseInheritDoc = function (token) {
        // Check to make sure the API definition reference is at most one string
        var tokenChunks = token.text.split(' ');
        if (tokenChunks.length > 1) {
            this.reportError('Too many parameters for @inheritdoc inline tag.' +
                'The format should be {@inheritdoc scopeName/packageName:exportName}. Extra parameters are ignored');
            return;
        }
        // Create the IApiDefinitionReference object
        // Deconstruct the API reference expression 'scopeName/packageName:exportName.memberName'
        var apiDefinitionRef = ApiDocumentation.parseApiReferenceExpression(token.text, this.reportError);
        // if API reference expression is formatted incorrectly then apiDefinitionRef will be undefined
        if (!apiDefinitionRef) {
            return;
        }
        // Atempt to locate the apiDefinitionRef
        var inheritedDoc = this.docItemLoader.getItem(apiDefinitionRef);
        // If no IDocItem found then nothing to inherit
        // But for the time being set the summary to a text object
        if (!inheritedDoc) {
            var textDocItem = {
                kind: 'textDocElement',
                value: "See documentation for " + tokenChunks[0]
            };
            this.summary = [textDocItem];
            return;
        }
        // inheritdoc found, copy over IDocBase properties
        this.summary = inheritedDoc.summary;
        this.remarks = inheritedDoc.remarks;
        // Copy over detailed properties if neccessary
        // Add additional cases if needed
        switch (inheritedDoc.kind) {
            case 'function':
                this.parameters = inheritedDoc.parameters;
                this.returnsMessage = inheritedDoc.returnValue.description;
                break;
        }
        // Check if inheritdoc is depreacted
        // We need to check if this documentation has a deprecated message
        // but it may not appear until after this token.
        this.isDocInheritedDeprecated = inheritedDoc.deprecatedMessage.length > 0 ? true : false;
    };
    ApiDocumentation.prototype._parseParam = function (tokenizer) {
        var paramDescriptionToken = tokenizer.getToken();
        if (!paramDescriptionToken) {
            this.reportError('@param tag missing required description');
            return;
        }
        var hyphenIndex = paramDescriptionToken ? paramDescriptionToken.text.indexOf('-') : -1;
        if (hyphenIndex < 0) {
            this.reportError('No hyphens found in the @param line. ' +
                'There should be a hyphen between the parameter name and its description.');
            return;
        }
        else {
            var name_1 = paramDescriptionToken.text.slice(0, hyphenIndex).trim();
            var comment = paramDescriptionToken.text.substr(hyphenIndex + 1).trim();
            if (!comment) {
                this.reportError('@param tag requires a description following the hyphen');
                return;
            }
            var commentTextElement = DocElementParser_1.default.makeTextElement(comment);
            // Full param description may contain additional Tokens (Ex: @link)
            var remainingElements = DocElementParser_1.default.parse(tokenizer, this.reportError);
            var descriptionElements = [commentTextElement].concat(remainingElements);
            var paramDocElement = {
                name: name_1,
                description: descriptionElements
            };
            return paramDocElement;
        }
    };
    ApiDocumentation.prototype._reportBadJSDocTag = function (token) {
        var supportsRegular = ApiDocumentation._allowedRegularJsdocTags.indexOf(token.tag) >= 0;
        var supportsInline = ApiDocumentation._allowedInlineJsdocTags.indexOf(token.tag) >= 0;
        if (!supportsRegular && !supportsInline) {
            this.reportError("Unknown JSDoc tag \"" + token.tag + "\"");
            return;
        }
        if (token.type === Token_1.TokenType.Inline && !supportsInline) {
            this.reportError("The JSDoc tag \"" + token.tag + "\" must not use the non-inline syntax (no curly braces)");
            return;
        }
        if (token.type === Token_1.TokenType.Tag && !supportsRegular) {
            this.reportError("The JSDoc tag \"" + token.tag + "\" must use the inline syntax (with curly braces)");
            return;
        }
        this.reportError("The JSDoc tag \"" + token.tag + "\" is not supported in this context");
        return;
    };
    ApiDocumentation.prototype._checkInheritDocStatus = function () {
        if (this.isDocInherited) {
            this.reportError('Cannot provide additional JSDoc tags if @inheritdoc tag is present');
        }
    };
    return ApiDocumentation;
}());
// For guidance about using these tags, please see this document:
// https://onedrive.visualstudio.com/DefaultCollection/SPPPlat/_git/sp-client
//    ?path=/common/docs/ApiPrinciplesAndProcess.md
ApiDocumentation._allowedRegularJsdocTags = [
    // (alphabetical order)
    '@alpha',
    '@beta',
    '@betadocumentation',
    '@internal',
    '@internalremarks',
    '@param',
    '@preapproved',
    '@public',
    '@returns',
    '@see',
    '@summary',
    '@deprecated',
    '@readonly',
    '@remarks'
];
ApiDocumentation._allowedInlineJsdocTags = [
    // (alphabetical order)
    '@inheritdoc',
    '@link'
];
/**
 * Match JsDoc block tags and inline tags
 * Example "@a @b@c d@e @f {whatever} {@link a} { @something } \@g" => ["@a", "@f", "{@link a}", "{ @something }"]
 */
ApiDocumentation._jsdocTagsRegex = /{\s*@(\\{|\\}|[^{}])*}|(?:^|\s)(\@[a-z_]+)(?=\s|$)/gi;
/**
 * Splits an API reference expression into two parts, first part is the scopename/packageName and
 * the second part is the exportName.memberName.
 */
ApiDocumentation._packageRegEx = /^([^:]*)\:(.*)$/;
/**
 * Splits the exportName.memberName into two respective parts.
 */
ApiDocumentation._memberRegEx = /^([^.|:]*)(?:\.(\w+))?$/;
/**
 * Used to ensure that the export name contains only text characters.
 */
ApiDocumentation._exportRegEx = /^\w+/;
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = ApiDocumentation;

//# sourceMappingURL=ApiDocumentation.js.map
