/* tslint:disable:no-bitwise */
/* tslint:disable:no-constant-condition */
"use strict";
var ts = require("typescript");
var ApiDocumentation_1 = require("./ApiDocumentation");
/**
 * ApiItem is an abstract base that represents TypeScript API definitions such as classes,
 * interfaces, enums, properties, functions, and variables.  Rather than directly using the
 * abstract syntax tree from the TypeScript Compiler API, we use ApiItem to extract a
 * simplified tree which correponds to the major topics for our API documentation.
 */
var ApiItem = (function () {
    function ApiItem(options) {
        this.reportError = this.reportError.bind(this);
        this.declaration = options.declaration;
        this._errorNode = options.declaration;
        this.warnings = [];
        this.analyzer = options.analyzer;
        this.declarationSymbol = options.declarationSymbol;
        this.exportSymbol = options.exportSymbol || this.declarationSymbol;
        this.name = this.exportSymbol.name || '???';
        this.typeChecker = this.analyzer.typeChecker;
        this.documentation = new ApiDocumentation_1.default(this, options.analyzer.docItemLoader, this.reportError);
    }
    /**
     * Return the compiler's underlying Declaration object
     * @todo Generally ApiItem classes don't expose ts API objects; we should add
     *       an appropriate member to avoid the need for this.
     */
    ApiItem.prototype.getDeclaration = function () {
        return this.declaration;
    };
    /**
     * Return the compiler's underlying Symbol object that contains semantic information about the item
     * @todo Generally ApiItem classes don't expose ts API objects; we should add
     *       an appropriate member to avoid the need for this.
     */
    ApiItem.prototype.getDeclarationSymbol = function () {
        return this.declarationSymbol;
    };
    /**
     * Whether this APiItem should have documentation or not.  If false, then
     * ApiItem.missingDocumentation will never be set.
     */
    ApiItem.prototype.shouldHaveDocumentation = function () {
        return true;
    };
    /**
     * This traverses any type aliases to find the original place where an item was defined.
     * For example, suppose a class is defined as "export default class MyClass { }"
     * but exported from the package's index.ts like this:
     *
     *    export { default as _MyClass } from './MyClass';
     *
     * In this example, calling followAliases() on the _MyClass symbol will return the
     * original definition of MyClass, traversing any intermediary places where the
     * symbol was imported and re-exported.
     */
    ApiItem.prototype.followAliases = function (symbol) {
        var current = symbol;
        while (true) {
            if (!(current.flags & ts.SymbolFlags.Alias)) {
                break;
            }
            var currentAlias = this.typeChecker.getAliasedSymbol(current);
            if (!currentAlias || currentAlias === current) {
                break;
            }
            current = currentAlias;
        }
        return current;
    };
    /**
     * Reports an error through the ApiErrorHandler interface that was registered with the Analyzer,
     * adding the filename and line number information for the declaration of this ApiItem.
     */
    ApiItem.prototype.reportError = function (message) {
        this.analyzer.reportError(message, this._errorNode.getSourceFile(), this._errorNode.getStart());
    };
    /**
      * Adds a warning to the ApiItem.warnings list.  These warnings will be emtted in the API file
      * produced by ApiFileGenerator.
      */
    ApiItem.prototype.reportWarning = function (message) {
        this.warnings.push(message);
    };
    return ApiItem;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = ApiItem;

//# sourceMappingURL=ApiItem.js.map
