/// <reference types="mocha" />
"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var chai_1 = require("chai");
var ts = require("typescript");
var path = require("path");
var Analyzer_1 = require("../../Analyzer");
var ApiDocumentation_1 = require("../ApiDocumentation");
/* tslint:disable:no-function-expression - Mocha uses a poorly scoped "this" pointer */
var capturedErrors = [];
function testErrorHandler(message, fileName, lineNumber) {
    capturedErrors.push({ message: message, fileName: fileName, lineNumber: lineNumber });
}
var analyzer = new Analyzer_1.default(testErrorHandler);
var inputFolder = './testInputs/example2';
var myDocumentedClass;
/**
 * Dummy class wrapping ApiDocumentation to test its protected methods
 */
var TestApiDocumentation = (function (_super) {
    __extends(TestApiDocumentation, _super);
    function TestApiDocumentation() {
        return _super.call(this, myDocumentedClass, analyzer.docItemLoader, function (msg) { return; }) || this;
    }
    return TestApiDocumentation;
}(ApiDocumentation_1.default));
// Run the analyzer once to be used by unit tests
analyzer.analyze({
    compilerOptions: {
        target: ts.ScriptTarget.ES5,
        module: ts.ModuleKind.CommonJS,
        moduleResolution: ts.ModuleResolutionKind.NodeJs,
        rootDir: inputFolder
    },
    entryPointFile: path.join(inputFolder, 'index.ts')
});
myDocumentedClass = analyzer.package.getSortedMemberItems()
    .filter(function (apiItem) { return apiItem.name === 'MyDocumentedClass'; })[0];
describe('ApiDocumentation tests', function () {
    this.timeout(10000);
    describe('ApiDocumentation internal methods', function () {
        var apiDoc = new TestApiDocumentation();
        var apiReferenceExpr;
        var expected;
        var actual;
        it('_parseApiReferenceExpression() with scope name', function () {
            apiReferenceExpr = '@microsoft/sp-core-library:Guid';
            expected = {
                scopeName: '@microsoft',
                packageName: 'sp-core-library',
                exportName: 'Guid',
                memberName: ''
            };
            actual = ApiDocumentation_1.default.parseApiReferenceExpression(apiReferenceExpr, apiDoc.reportError);
            chai_1.assert.equal(expected.scopeName, actual.scopeName);
            chai_1.assert.equal(expected.packageName, actual.packageName);
            chai_1.assert.equal(expected.exportName, actual.exportName);
            chai_1.assert.equal(expected.memberName, actual.memberName);
        });
        it('_parseApiReferenceExpression() without scope name', function () {
            apiReferenceExpr = 'sp-core-library:Guid';
            expected = { scopeName: '', packageName: 'sp-core-library', exportName: 'Guid', memberName: '' };
            actual = ApiDocumentation_1.default.parseApiReferenceExpression(apiReferenceExpr, apiDoc.reportError);
            chai_1.assert.equal(expected.scopeName, actual.scopeName);
            chai_1.assert.equal(expected.packageName, actual.packageName);
            chai_1.assert.equal(expected.exportName, actual.exportName);
            chai_1.assert.equal(expected.memberName, actual.memberName);
        });
        it('_parseApiReferenceExpression() without scope name and with member name', function () {
            apiReferenceExpr = 'sp-core-library:Guid.equals';
            expected = { scopeName: '', packageName: 'sp-core-library', exportName: 'Guid', memberName: 'equals' };
            actual = ApiDocumentation_1.default.parseApiReferenceExpression(apiReferenceExpr, apiDoc.reportError);
            chai_1.assert.equal(expected.scopeName, actual.scopeName);
            chai_1.assert.equal(expected.packageName, actual.packageName);
            chai_1.assert.equal(expected.exportName, actual.exportName);
            chai_1.assert.equal(expected.memberName, actual.memberName);
        });
        it('_parseApiReferenceExpression() without scope name and invalid memberName', function () {
            // (Error #6)
            apiReferenceExpr = 'sp-core-library:Guid:equals';
            var caughtError = false;
            try {
                actual = ApiDocumentation_1.default.parseApiReferenceExpression(apiReferenceExpr, apiDoc.reportError);
            }
            catch (error) {
                caughtError = true;
            }
            chai_1.assert.equal(caughtError, true);
        });
    });
    describe('Documentation Parser Tests', function () {
        it('Should report errors', function () {
            /**
             * To view the expected errors see:
             * - testInputs/example2/folder/MyDocumentedClass (9 errors)
             * - the test in this file '_parseApiReferenceExpression() without scope name and invalid memberName' (1 error)
             */
            chai_1.assert.equal(capturedErrors.length, 10);
        });
        it('Should parse API tag', function () {
            var expecedApiTag = ApiDocumentation_1.ApiTag.Public;
            var actualDoc = myDocumentedClass ? myDocumentedClass.documentation : undefined;
            chai_1.assert.isObject(actualDoc);
            chai_1.assert.equal(actualDoc.apiTag, expecedApiTag);
        });
    });
});

//# sourceMappingURL=ApiDocumentation.test.js.map
